*********************************************************************
*
*     Program: SOLITARE.PRG
*      Author: Jim Denney
*			   Axis Software
*			   921 North Hill
*			   Springfield, Illinois  62702
*			   [217] 789-7594
*
*   Completed: Feb 27, 1991
*
* Description: This program simulates the card game solitare. The
*              cards are laid out in seven rows, a row being 1 to
*              7 cards at the start. There is also a stack (all undealt
*              cards are kept there) and the top where all cards of the
*              same suit are kept. The object of the game is to place
*              the cards on their resective suits in sequential order.
*              Cards may be placed there from the stack or from the rows.
*              Cards are placed on the rows in reverse sequentail order
*              and alternating colors. If an illegal move is attempted,
*              you will receive a warning. You may change the warning
*              messages - their present contents help break the
*              boredom of debugging.
*
*              This program can be played using either a mouse or the
*              keyboard. To use the keyboard, numbers stand for the
*              rows of cards, "S" stands for the stack of undealt cards,
*              and "T" stands for the top.
*
*
*              Some notes on the program:
*
*              1. This program uses only memory variables, no databases.
*                 We initialize two arrays at first. One is used to help
*                 shuffle the cards then released.
*
*              2. Shuufling could probably use improvement. The xbase
*                 language has no decent random function. To increase the
*                 amount of shuffling, increase the number of times the
*                 DO - WHILE loop is executed in the shuffle procedure.
*
*              3. The program is not bug free, but it is close. I make no
*                 guarantees as to the quality of play. I will say you
*                 have a better chance of winning than at the Foxpro
*                 puzzle game.
*
*              4. When using a mouse, the program looks for the column
*                 the cursor is clicked on. If it is past the rows of
*                 cards, it checks the row to see if it should chose
*                 the stack or the top.
*
*              5. Scoring is simple. You receive five points for each
*                 cards you place on top. I did not want to get to
*                 complicated here. You may modify this if you choose.
*
*              6. You can go as many times through the stack of undealt
*                 cards as you wish with no penalty. This also can be
*                 changed to suit your needs.
*
**********************************************************************

SET ESCAPE ON
SET TALK OFF
DEFINE WINDOW game_wind FROM 0, 0 TO 24, 79 NONE
ACTIVATE WINDOW game_wind
ex_stat = SET('EXACT')
SET EXACT OFF

* The following DIMENSION statements create three arrays. Stacks are
* the seven rows of cards dealt out, deck is the deck of 52 cards
* started with, cards is the array used for suffling.

DIMENSION stacks(7, 19), deck(52), cards(52)

* Of the following variables, the top_# variables contain the suits
* of cards at the top. Row# variables contain the total number of
* cards in a row and down# variables are the cards lying face down.

STORE 0 TO kount, suit, pile, top_1, top_2, top_3, top_4, knt, dealt,;
      row1, row2, row3, row4, row5, row6, row7, down1, down2, down3, down4,;
      down5, down6, down7, score
STORE '' TO char, char2, cnt,str_1,str_2
STORE .T. TO good_sel, swap, rat

* Define special keys
mouse =  151
ESC   = 27
help  = 28
key   =  0

* DEFINE SUITS OF CARDS
top_1 = 2
FOR suit = 1 to 4
    top_1 = top_1 + 1
    char = CHR(top_1)
    FOR kount = 1 to 13
        cards((13 * (suit-1)) + kount) = STR(kount) + ' ' + char
        DO CASE
           CASE kount = 1
              cards((13 * (suit-1)) + kount) =  char + ' A ' + char
           CASE kount = 11
              cards((13 * (suit-1)) + kount) =  char + ' J ' + char
           CASE kount = 12
              cards((13 * (suit-1)) + kount) =  char + ' Q ' + char
           CASE kount = 13
              cards((13 * (suit-1)) + kount) =  char + ' K ' + char
           CASE kount = 10
              cards((13 * (suit-1)) + kount) =  char + '10 ' + char
           OTHERWISE
              cards((13 * (suit-1)) + kount) =  char + ' ' + STR(kount,1) + ' ' + char
        ENDCASE
    ENDFOR
ENDFOR
top_1 = 0

* Set up the playing area now.
DO draw_scr
DO shuffle
DO deal_cards

DO WHILE .T.
   @ 22, 62 SAY score PICTURE '999'
   DO show_stack
   key=INKEY(0, "HM")      && Check for mouse click and hide cursor
   IF CHR(key) $ 'dD'
      * If they pressed the "D" key they want to start a new game.
      STORE 0 TO kount, suit, pile, top_1, top_2, top_3, top_4, knt, dealt,;
                 row1, row2, row3, row4, row5, row6, row7, down1, down2,;
                 down3, down4, down5, down6, down7, score
      DO draw_scr
      DO shuffle
      DO deal_cards
      LOOP
   ENDIF
   
   IF key = ESC
      * They selected to quit playing.
      EXIT
   ENDIF
   
   IF key = help
      DO disp_help
      LOOP
   ENDIF
   
   rat = .F.
   IF key = mouse
      DO rodent
      rat = .T.
   ELSE
      * If not one of the keys we need, try again.
      IF !(BETWEEN(key, 49, 55) OR BETWEEN(key, 83, 84) OR;
           BETWEEN(key, 115, 116) OR key = 78 OR key = 110)
         LOOP
      ENDIF
   ENDIF
   
   * Must be a valid key, lets get the second key next.
   DO get_inpt
   IF !good_sel      && They either has large clumsy fingers or don't know
     good_sel = .T.  && how to play the game.
     LOOP
   ENDIF
   
   IF dealt >= 52    
      * No cards left on the stack.
      * Could be they got lucky and won the game. If no cards on the rows
      * they won.
      IF row1 + row2 + row3 + row4 + row5 + row6 + row7 = 0
         DO winner
      ENDIF
   ENDIF
ENDDO

SHOW WINDOWS ALL
IF ex_stat = 'ON'
   SET EXACT ON
ENDIF
RELEASE WINDOW game_wind
RETURN

**************************************************************************
PROCEDURE get_inpt
**************************************************************************
* Get input from the user and decipher. If a valid choice, display the
* choices and execute the proper procedures.

IF !rat   && No mouse click, get keyboard character.
   char = CHR(key)
ENDIF

DO CASE
   CASE char < '8'                    && A number was entered.
      @ 14, 55 SAY 'ROW ' + char
      IF down&char < 0
         down&char = 0
      ENDIF

   CASE char = 'S' OR char = 's'    && The stack was chosen.
      char = 'S'
      @ 14, 55 SAY 'STACK'

   CASE char = 'N' OR char = 'n'    && Deal the next card
      DO deal_stack
      RETURN
ENDCASE

CLEAR TYPEAHEAD

IF rat
   timelimit =  SECONDS() + _dblclick  && I don't know why, it was
   key = 0                             && in the book. It works, so I
   CLEAR TYPEAHEAD                     && ain't gonna mess with it.
   CLEAR PROMPT
   CLEAR GETS
   key=INKEY(0, "HM")
   DO rodent
ELSE
   CLEAR TYPEAHEAD
   char2 = CHR(INKEY(key))            && Get the 2nd key from the keyboard.
ENDIF

IF char2 $ 'tT'
   * Convert to upper
   char2 = 'T'
ELSE
   * If no cards on the row, make sure it is set to 0.
   IF down&char2 < 0
      down&char2 = 0
   ENDIF
ENDIF

DO CASE
   CASE char = 'S' AND char2 = 'T'
      * Move from the stack to the top row of suits.
      char = 'S'
      char2 = 'T'
      @ 14, 64 SAY ' TOP  '
      DO val_top
      IF swap
         DO stk_top
      ELSE
         RETURN
      ENDIF
      
   CASE char = 'S' AND char2 < '8'
      * Move from the stack to one of the rows.
      char = 'S'
      @ 14, 64 SAY 'ROW ' + char2
      DO stk_row
      
   CASE char < '8' AND char2 < '8'
      * Move from a row to another row.
      @ 14, 64 SAY 'ROW ' + char2
      IF row&char < 0
         WAIT "Selected row is empty and the high card isn't a king, try again..." WINDOW
         RETURN
      ENDIF
      DO row_row
      
   CASE char < '8' AND char2 = 'T'
      * Move the bottom card from a row to the top row of suits.
      char2 = 'T'
      @ 14, 64 SAY ' TOP  '
      DO row_top
      
   OTHERWISE
      * Don't know what happened, but it wasn't permitted.
      @ 24, 0 SAY CHR(7)
      @ 14, 55 SAY '     '
      @ 14, 64 SAY '     '
ENDCASE
RETURN

**************************************************************************
PROCEDURE row_row
**************************************************************************
* Check to see if we can move cards from one row to another.

STORE '' TO str_1, str_2
str_1 = stacks(VAL(char), down&char + 1)
IF row&char2 > 0
   str_2 = stacks(VAL(char2), row&char2)
ELSE
   str_2 = '     '
ENDIF

IF SUBSTR(str_1, 3, 1) = 'K'   
   * If it is a king, it must go to an empty row.
   IF row&char2 > 0
      WAIT "Can't move the cards, the row must be empty." WINDOW NOWAIT
      RETURN
   ENDIF
ENDIF

* The following IF statement checks to see the cards are alternating colors.

IF ((LEFT(str_1, 1) = '' OR LEFT(str_1, 1) = '') AND;
    (LEFT(str_2, 1) = '' OR LEFT(str_2, 1) = '')) OR;
   ((LEFT(str_2, 1) = '' OR LEFT(str_2, 1) = '') AND;
    (LEFT(str_1 ,1) = '' OR LEFT(str_1, 1) = '')) OR;
   (SUBSTR(str_1, 3, 1) = 'K' AND row&char2 = 0)
   swap = .T.
   DO chk_move
   IF swap
      pile = 1

      * If we are here, the cards need to be moved. Lets move all face up
      * cards to the second row chosen

      FOR kount = down&char + 1 TO row&char
         row&char2 = row&char2 + 1
         stacks(VAL(char2), row&char2) = stacks(VAL(char), down&char + pile)
         stacks(VAL(char), down&char + pile) = '     '
         pile = pile + 1
      ENDFOR

      IF down&char > -1
         down&char = down&char - 1
         row&char = down&char + 1
      ENDIF
      DO redraw
   ENDIF
ELSE
   WAIT 'Cards must be played in alternating colors, try again...' WINDOW
   good_sel = .F.
ENDIF
RETURN

**************************************************************************
PROCEDURE row_top
**************************************************************************

str_1 = stacks(VAL(char), row&char)
DO val_top

IF swap
   stacks(VAL(char), row&char) = '     ' && Make the last card blank
   row&char = row&char - 1               && Subtract 1 from stack of cards
   IF row&char = down&char               && If no cards up reduce the number
      down&char = down&char - 1          && of cards uncovered so redraw
   ENDIF                                 && will flip the last card over.
   DO redraw
ENDIF
RETURN

**************************************************************************
PROCEDURE stk_row
**************************************************************************
* Here we move a card from the stack to a row of cards

IF char = 'S' AND char2 < '9'
   DO  chk_move
   IF !swap
      RETURN
   ENDIF
   str_1 = deck(dealt)
   dealt = dealt + 1
   IF deck(dealt) = '' OR deck(dealt) = ''
      @  9, 58 SAY deck(dealt) COLOR R/W
      @ 11, 58 SAY deck(dealt) COLOR R/W
   ELSE
      @  9, 58 SAY deck(dealt) COLOR N/W
      @ 11, 58 SAY deck(dealt) COLOR N/W
   ENDIF
   row&char2 = row&char2 + 1
   stacks(VAL(char2), row&char2) = str_1
   DO redraw
ENDIF
RETURN

**************************************************************************
PROCEDURE stk_top
**************************************************************************
* Here we moves a card to one of the top four piles.

IF char2 = 'T'
   IF char = 'S'
      IF dealt > 52
         dealt = 52
      ENDIF
      str_1 = deck(dealt)
      dealt = dealt + 1
   ENDIF

   DO CASE
      CASE LEFT(str_1, 1) = ''
         @ 3, 46 SAY str_1 COLOR N/W
         @ 5, 46 SAY str_1 COLOR N/W

      CASE LEFT(str_1, 1) = ''
         @ 3, 54 SAY str_1 COLOR R/W
         @ 5, 54 SAY str_1 COLOR R/W

      CASE LEFT(str_1, 1) = ''
         @ 3, 62 SAY str_1 COLOR N/W
         @ 5, 62 SAY str_1 COLOR N/W

      CASE LEFT(str_1, 1) = ''
         @ 3, 70 SAY str_1 COLOR R/W
         @ 5, 70 SAY str_1 COLOR R/W
   ENDCASE
ENDIF
RETURN

**************************************************************************
PROCEDURE deal_stack
**************************************************************************
* Display the next card on the stack.

t_1 = ''
DO CASE
   CASE dealt < 52
      t_1 = deck(52)                    && Simple procedure, just make the
      FOR kount = 52 TO dealt STEP -1   && first card the last and move
         deck(kount) = deck(kount - 1)  && all the others down one number.
      ENDFOR
      deck(dealt) = t_1

   CASE dealt = 52
      WAIT 'Last card...' WINDOW

   OTHERWISE
      WAIT 'There are no cards displayed.' WINDOW NOWAIT
ENDCASE
RETURN

**************************************************************************
                           PROCEDURE chk_move
**************************************************************************
* Checks to see if moves are valid by assigning each card a number. A valid
* move has the fisrt face-up card in from the row being moved as one less
* than the value of the card it is being placed upon.

num = 0
num2 = 0
swap = .F.
IF char = 'S'
   str_1 = deck(dealt)
ELSE
   str_1 = stacks(VAL(char), down&char + 1)
ENDIF
IF row&char2 > 0
   str_2 = stacks(VAL(char2), row&char2)
ELSE
   str_2 = '    '
ENDIF

DO CASE
   CASE SUBSTR(str_1, 3, 1) = 'A'
      num = 1

   CASE SUBSTR(str_1, 3, 1) = 'J'
      num = 11

   CASE SUBSTR(str_1, 3, 1) = 'Q'
      num = 12

   CASE SUBSTR(str_1, 3, 1) = 'K'
      num = 13

   OTHERWISE
      num = VAL(SUBSTR(str_1, 2, 2))
ENDCASE
DO CASE
   CASE SUBSTR(str_2, 3, 1) = 'A'
      num2 = 1

   CASE SUBSTR(str_2, 3, 1) = 'J'
      num2 = 11

   CASE SUBSTR(str_2, 3, 1) = 'Q'
      num2 = 12

   CASE SUBSTR(str_2, 3, 1) = 'K'
      num2 = 13

   OTHERWISE
      num2 = VAL(SUBSTR(str_2, 2, 2))
ENDCASE

IF num2 = num + 1 OR (num = 13 AND row&char2 = 0)
   swap = .T.
ENDIF

IF !swap
   WAIT "I ain't moving those cards, you want to cheat play another game." WINDOW NOWAIT
ENDIF
RETURN

**************************************************************************
PROCEDURE val_top
**************************************************************************
* Check to see if the last card on the row or the card on the stack is
* a vaild card to move to the top. Again, we assign a number to the cards
* and check to see if it is one more than the top card of the suit it
* belongs to presently on the top pile.

num = 0
swap = .F.
IF char = 'S'
   str_1 = deck(dealt)
ENDIF
DO CASE
   CASE SUBSTR(str_1, 3, 1) = 'A'
      num = 1

   CASE SUBSTR(str_1, 3, 1) = 'J'
      num = 11

   CASE SUBSTR(str_1, 3, 1) = 'Q'
      num = 12

   CASE SUBSTR(str_1, 3, 1) = 'K'
      num = 13

   OTHERWISE
      num = VAL(SUBSTR(str_1, 2, 2))
ENDCASE
DO CASE
   CASE LEFT(str_1, 1) = ''
      IF num = top_1 + 1
         @ 3, 46 SAY str_1 COLOR N/W
         @ 5, 46 SAY str_1 COLOR N/W
         top_1 = num
         swap = .T.
      ENDIF

   CASE LEFT(str_1, 1) = ''
      IF num = top_2 + 1
         @ 3, 54 SAY str_1 COLOR R/W
         @ 5, 54 SAY str_1 COLOR R/W
         top_2 = num
         swap = .T.
      ENDIF

   CASE LEFT(str_1, 1) = ''
      IF num = top_3 + 1
         @ 3, 62 SAY str_1 COLOR N/W
         @ 5, 62 SAY str_1 COLOR N/W
         top_3 = num
         swap = .T.
      ENDIF

   CASE LEFT(str_1, 1) = ''
      IF num = top_4 + 1
         @ 3, 70 SAY str_1 COLOR R/W
         @ 5, 70 SAY str_1 COLOR R/W
         top_4 = num
         swap = .T.
      ENDIF
ENDCASE

IF char < '8'
   IF row&char = 0
      swap = .T.
   ENDIF
ENDIF

IF !swap
   WAIT "Transfer can't be made, check your cards." WINDOW NOWAIT
ELSE
   score = score + 5
ENDIF
RETURN

**************************************************************************
PROCEDURE redraw
**************************************************************************
* Just redraws the rows of cards.

IF char < '8'
   pos = VAL(char) - 1
   @ 0, pos * 6 CLEAR TO 24, pos * 6 + 5
   IF row&char > 0
      FOR x = 1 TO row&char
         cnt = str(x,1)
         IF x <= down&char              && If there are cards face down.
            IF x / 2 = INT(x / 2)       && Even numbered rows start with
               @ x, pos * 6 SAY '' && darker cards.
            ELSE
               @ x, pos * 6 SAY ''
            ENDIF
         ELSE
            * Draw all the cards face up on the stack
            IF stacks(VAL(char), x) = '' OR stacks(VAL(char), x) = ''
               @ down&char + x - down&char, pos * 6 SAY stacks(VAL(char), x) COLOR R/W
            ELSE
               @ down&char + x - down&char, pos * 6 SAY stacks(VAL(char), x) COLOR N/W
            ENDIF
         ENDIF
      ENDFOR
   ENDIF
   
   * Draw the last card face up on the stack and put the row number on it.
   IF row&char > 0
      IF stacks(VAL(char), row&char) = '' OR;
         stacks(VAL(char), row&char) = ''
         @  row&char, pos * 6 SAY stacks(VAL(char), row&char) COLOR R/W
         @  row&char + 1, pos * 6 SAY 'Row ' + char COLOR N/W
         @  row&char + 2, pos * 6 SAY stacks(VAL(char), row&char) COLOR R/W
      ELSE
         @  row&char, pos * 6 SAY stacks(VAL(char), row&char) COLOR N/W
         @  row&char + 1, pos * 6 SAY 'Row ' + char COLOR N/W
         @  row&char + 2, pos * 6 SAY stacks(VAL(char), row&char) COLOR N/W
      ENDIF
   ENDIF
ENDIF

* This is a copy of the first half of the procedure modified to work with
* a two instead of a 1. I know, it could have been put in a loop and done
* with macros, but this was easier. Besides, speed is not an issue here.

IF char2 < '8'
   pos = VAL(char2) - 1
   @ 0, pos * 6 CLEAR TO 24, pos * 6 + 5
   IF row&char2 > 0
      FOR x = 1 TO row&char2
         cnt = str(x,1)
         IF x <= down&char2
            IF x / 2 = INT(x / 2)
               @ x, pos * 6 SAY ''
            ELSE
               @ x, pos * 6 SAY ''
            ENDIF
         ELSE
            IF stacks(VAL(char2), x) = '' OR stacks(VAL(char2), x) = ''
               @ down&char2 + x - down&char2, pos * 6 SAY stacks(VAL(char2), x) COLOR R/W
            ELSE
               @ down&char2 + x - down&char2, pos * 6 SAY stacks(VAL(char2), x) COLOR N/W
            ENDIF
         ENDIF
      ENDFOR
   ENDIF
   
   IF row&char2 > 0
      IF stacks(VAL(char2), row&char2) = '' OR;
         stacks(VAL(char2), row&char2) = ''
         @  row&char2, pos * 6 SAY stacks(VAL(char2), row&char2) COLOR R/W
         @  row&char2 + 1, pos * 6 SAY 'Row ' + char2 COLOR N/W
         @  row&char2 + 2, pos * 6 SAY stacks(VAL(char2), row&char2) COLOR R/W
      ELSE
         @  row&char2, pos * 6 SAY stacks(VAL(char2), row&char2) COLOR N/W
         @  row&char2 + 1, pos * 6 SAY 'Row ' + char2 COLOR N/W
         @  row&char2 + 2, pos * 6 SAY stacks(VAL(char2), row&char2) COLOR N/W
      ENDIF
   ENDIF
ENDIF
RETURN

**************************************************************************
PROCEDURE show_stack
**************************************************************************
* Show the top card of the stack. If none, show none.

IF dealt > 52
   @  9, 58 SAY '     '
   @ 11, 58 SAY '     '
   RETURN
ENDIF

IF deck(dealt) = '' OR deck(dealt) = ''
   @  9, 58 SAY deck(dealt) COLOR R/W
   @ 11, 58 SAY deck(dealt) COLOR R/W
ELSE
   @  9, 58 SAY deck(dealt) COLOR N/W
   @ 11, 58 SAY deck(dealt) COLOR N/W
ENDIF
RETURN

**************************************************************************
PROCEDURE draw_scr
**************************************************************************

CLEAR
? '          TOP  ͻ'
? '         ͻ ͻ ͻ ͻ '
? '             '
? '              SPADE HEART CLUBS DMNDS '
? '                       '
? '                        ͼ ͼ ͼ ͼ '
? '                            ͼ'
? '                                               ͻ'
? '                                                    '
? '                                                         STACK'
? '                                                         '
? '                                                         ͼ'
? '                                                 ͻ'
? '                                                 Move       To       '
? '                                                 ͼ'
? '                                                   ͻ'
? '                                                    [F-1] -  HELP  '
? '                                                    [ESC] -  EXIT  '
? '                                                     [D]  -  DEAL  '
? '                                                   ͼ'
? '                                                     ͻ'
? '                                                     Score:      '
? '                                                     ͼ'
@ 4, 46 SAY 'SPADE' COLOR N/W
@ 4, 54 SAY 'HEART' COLOR N/W
@ 4, 62 SAY 'CLUBS' COLOR N/W
@ 4, 70 SAY 'DMNDS' COLOR N/W
@10, 58 SAY 'STACK' COLOR N/W
RETURN

**************************************************************************
PROCEDURE SHUFFLE
**************************************************************************
* This routine takes cards at random from the first array, places them in
* the second array and decreases the maximum size of the random number that
* can be generated. After all the cards are sawpped, they are put back in
* the first array and the procedure is repeated. It is not a good routine,
* but we don't have good tools to work with. 
* -- When all you have is a nail, everything resembles a hammer. --

r_pet = 0
DO WHILE r_pet < 10
   t_knt = 0
   FOR kount = 52 TO 1 STEP -1
      tmp_var = cards(kount)
      t_knt = INT(RAND(-1) * kount+1)
      deck(kount) = cards(t_knt)
      cards(kount) = cards(t_knt)
      cards(t_knt) = tmp_var
   ENDFOR
   FOR kount = 1 to 52
      cards(kount) = deck(kount)
   ENDFOR
   r_pet = r_pet + 1
ENDDO
RETURN

*******************************************************************
PROCEDURE disp_cards
*******************************************************************
* Just show the last card of the rows. Called at start-up

FOR kount = 1 TO 7
   IF LEFT(stacks(kount, kount), 1) = '' OR;
      LEFT(stacks(kount, kount), 1) = ''
      @ kount, kount * 6 - 6 SAY stacks(kount, kount) COLOR R/W
      @ kount + 1, kount * 6 - 6 SAY 'Row ' + STR(kount,1) COLOR N/W
      @ kount + 2, kount * 6 - 6 SAY stacks(kount, kount) COLOR R/W
   ELSE
      @ kount, kount * 6 - 6 SAY stacks(kount, kount) COLOR N/W
      @ kount + 1, kount * 6 - 6 SAY 'Row ' + STR(kount,1) COLOR N/W
      @ kount + 2, kount * 6 - 6 SAY stacks(kount, kount) COLOR N/W
   ENDIF
ENDFOR

**************************************************************************
PROCEDURE disp_help
**************************************************************************
* Use a popup menu for a help screen. Simple - and it works.

DEFINE POPUP men_sel FROM 0, 18 SHADOW COLOR SCHEME 5
DEFINE BAR  1 OF men_sel PROMPT '                 KEYBOARD INSTRUCTIONS'
DEFINE BAR  2 OF men_sel PROMPT '[S] then Row #   - Moves cards from the stack to a row'
DEFINE BAR  3 OF men_sel PROMPT 'Row # then Row # - Moves cards from row to row'
DEFINE BAR  4 OF men_sel PROMPT 'Row # then [T]   - Moves card to top'
DEFINE BAR  5 OF men_sel PROMPT '[S] then [T]     - Moves card from stack to top'
DEFINE BAR  6 OF men_sel PROMPT '[N]              - Next card from the stack'
DEFINE BAR  7 OF men_sel PROMPT '' SKIP
DEFINE BAR  8 OF men_sel PROMPT '                   MOUSE INSTUCTIONS'
DEFINE BAR  9 OF men_sel PROMPT 'Move stack to row - Place cursor over stack and click,'
DEFINE BAR 10 OF men_sel PROMPT '                    then over desired row and click'
DEFINE BAR 11 OF men_sel PROMPT 'Move row to row   - Place cursor over first row and click'
DEFINE BAR 12 OF men_sel PROMPT '                    then over second row and click'
DEFINE BAR 13 OF men_sel PROMPT 'Move card to top  - Place cursor over stack and click,'
DEFINE BAR 14 OF men_sel PROMPT '                    then over desired row and click'
DEFINE BAR 15 OF men_sel PROMPT 'Move stack to top - Place cursor over first row and click'
DEFINE BAR 16 OF men_sel PROMPT '                    then over second row and click'
DEFINE BAR 17 OF men_sel PROMPT 'New card to stack - Place cusror on stack, click twice'
DEFINE BAR 18 OF men_sel PROMPT '' SKIP
DEFINE BAR 19 OF men_sel PROMPT 'Five points are awarded for each card placed on the top.' SKIP
DEFINE BAR 20 OF men_sel PROMPT '' SKIP
DEFINE BAR 21 OF men_sel PROMPT '                  Programmed by Jim Denney' SKIP
DEFINE BAR 22 OF men_sel PROMPT '                Axis Software  [217] 789-7594' SKIP

ON SELECTION POPUP men_sel DEACTIVATE POPUP men_sel
ACTIVATE POPUP men_sel
RETURN

**************************************************************************
PROCEDURE rodent
**************************************************************************
* Decipher those mouse clicks. We simply check the screen location to
* determine waht is being attempted.

row = MROW()
col = MCOL()
DO CASE
   CASE col < 6    && First row
      IF rat
         char2 = '1'
      ELSE
         char = '1'
      ENDIF

   CASE BETWEEN(col, 7, 12)
      IF rat
         char2 = '2'
      ELSE
         char = '2'
      ENDIF

   CASE BETWEEN(col, 13, 18)
      IF rat
         char2 = '3'
      ELSE
         char = '3'
      ENDIF

   CASE BETWEEN(col, 19, 24)
      IF rat
         char2 = '4'
      ELSE
         char = '4'
      ENDIF

   CASE BETWEEN(col, 25, 30)
      IF rat
         char2 = '5'
      ELSE
         char = '5'
      ENDIF

   CASE BETWEEN(col, 31, 36)
      IF rat
         char2 = '6'
      ELSE
         char = '6'
      ENDIF

   CASE BETWEEN(col, 37, 42)
      IF rat
         char2 = '7'
      ELSE
         char = '7'
      ENDIF

   OTHERWISE
      IF row > 7
         char = 'S'
         IF rat
            char = 'N'
            DO deal_stack
         ENDIF
      ELSE
         char2 = 'T'
      ENDIF
ENDCASE

IF char = 'S' AND char2 = 'S'
   char = 'N'
ENDIF
RETURN

******************************************************************
PROCEDURE winner
******************************************************************
* Display random boxes on the screen in random colors with a message.

m1 = ' *****WE HAVE A WINNER**** '
m2 = ' HEY EVERYBODY, LOOK HERE. '
m3 = ' I BET YOU ARE HAPPY NOW.  '
m4 = ' ----STRIKE UP THE BAND--- '
m5 = ' NOW YOU CAN REST IN PEACE '
m6 = ' WHO NEEDS WINDOWS ANYWAY? '

r_pete = .T.
CLEAR
kount = 1
DO WHILE r_pete
   row = INT(RAND(-1) * 23)
   col = INT(RAND() * 60)
   IF row > 21
      LOOP
   ENDIF
   IF col > 53
      LOOP
   ENDIF
   
   * Here we get our random colors for the boxes and text.
   cnt = STR(kount, 1)
   bg = INT(RAND(-1) * 8)
   fg = INT(RAND() * 8)
   bgc = IIF(bg = 1, 'N', IIF(bg = 2, 'B', IIF(bg = 3, 'GR', IIF(bg = 4, 'BG', IIF(bg = 5, 'G', IIF(bg = 6, 'RB', IIF(bg = 7, 'R', 'W')))))))
   fgc = IIF(fg = 1, 'W+', IIF(fg = 2, 'B+', IIF(fg = 3, 'GR+', IIF(fg = 4, 'BG+', IIF(fg = 5, 'G+', IIF(fg = 6, 'RB+', IIF(fg = 7, 'R+', 'W+' )))))))

   @ row, col TO row + 2, col + 28 COLOR &bgc/&fgc
   @ row + 1, col + 1 SAY m&cnt COLOR &bgc/&fgc
   IF INKEY() > 0
      r_pete = .F.
   ENDIF
   kount = kount + 1
   IF kount = 7
      kount = 1
   ENDIF
   FOR knt = 1 to 300
      * Delay loop
   ENDFOR
ENDDO
RETURN

******************************************************************
PROCEDURE deal_cards
******************************************************************
   * Deal the cards to the columns from the deck.
   FOR kount = 1 TO 7
      FOR knt = 1 TO kount
         dealt = dealt + 1
         stacks(kount, knt) = deck(dealt)
      ENDFOR
   ENDFOR

   dealt = dealt + 1
   FOR kount = 1 TO 7
      cnt = STR(kount, 1)
      row&cnt = kount
      down&cnt = kount - 1
   ENDFOR

   DO disp_cards
RETURN

* EOF SOLITARE.PRG
